﻿using System;
using System.Reflection.Emit;

namespace ModMaker.Lua.Parser.Items
{
    /// <summary>
    /// Defines a parse item that represents a label.
    /// </summary>
    public sealed class LabelItem : IParseStatement
    {
        /// <summary>
        /// Creates a new instance of LabelItem with the given name.
        /// </summary>
        /// <param name="name">The name of the label.</param>
        public LabelItem(string name)
        {
            if (name == null)
                throw new ArgumentNullException("name");

            this.Name = name;
            this.Label = null;
        }

        /// <summary>
        /// Gets or sets the name of the label.
        /// </summary>
        public string Name { get; set; }
        /// <summary>
        /// Gets or sets the debug info for this item.
        /// </summary>
        public Token Debug { get; set; }
        /// <summary>
        /// Gets or sets the user data for this object. This value is never
        /// modified by the default framework, but may be modified by other
        /// visitors.
        /// </summary>
        public object UserData { get; set; }
        /// <summary>
        /// Gets or sets the label that represents this object.
        /// </summary>
        public Label? Label { get; set; }

        /// <summary>
        /// Dispatches to the specific visit method for this item type.
        /// </summary>
        /// <param name="visitor">The visitor object.</param>
        /// <returns>The object returned from the specific IParseItemVisitor method.</returns>
        /// <exception cref="System.ArgumentNullException">If visitor is null.</exception>
        public IParseItem Accept(IParseItemVisitor visitor)
        {
            if (visitor == null)
                throw new ArgumentNullException("visitor");

            return visitor.Visit(this);
        }
    }
}